# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=C0330, g-bad-import-order, g-multiple-import

"""Instantiates concrete fetcher and builds fetching context."""

from media_fetching.sources import dbm, fake, file, googleads, sql, youtube

FETCHERS = {
  'youtube': (youtube.YouTubeFetchingParameters, youtube.Fetcher),
  'googleads': (googleads.GoogleAdsFetchingParameters, googleads.Fetcher),
  'file': (file.FileFetchingParameters, file.Fetcher),
  'bq': (sql.BigQueryFetchingParameters, sql.BigQueryFetcher),
  'sqldb': (sql.SqlAlchemyQueryFetchingParameters, sql.SqlAlchemyQueryFetcher),
  'fake': (fake.FakeFetchingParameters, fake.FakeFetcher),
  'dbm': (dbm.BidManagerFetchingParameters, dbm.Fetcher),
}
